/**
 * @file
 * Type representing a number
 *
 * @authors
 * Copyright (C) 2024 Pietro Cerutti <gahr@gahr.ch>
 *
 * @copyright
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MUTT_CONFIG_NUMBER_H
#define MUTT_CONFIG_NUMBER_H

struct Buffer;
struct ConfigSubset;
struct HashElem;

int number_he_toggle(struct ConfigSubset *sub, struct HashElem *he, struct Buffer *err);

#endif /* MUTT_CONFIG_NUMBER_H */

